﻿using Rhino;
using Rhino.Commands;
using Rhino.UI;
using System;
using System.Diagnostics;

namespace SamplePanel
{
  /// <summary>
  /// SamplePanel view model
  /// </summary>
  internal class SampleDocumentPanelViewModel : ViewModel
  {
    #region Static Event Handlers
    private static EventHandler<CommandEventArgs> g_begin_command;
    private static EventHandler<CommandEventArgs> g_end_command;
    #endregion

    /// <summary>
    /// Public constructor
    /// </summary>
    public SampleDocumentPanelViewModel(uint documentSerialNumber)
    {
      DocumentSerialNumber = documentSerialNumber;
      HookRhinoEvents();
    }

    #region Properties

    /// <summary>
    /// The current document runtime serial number
    /// </summary>
    public uint DocumentSerialNumber { get; private set; }

    /// <summary>
    /// The current document
    /// </summary>
    public RhinoDoc Document => RhinoDoc.FromRuntimeSerialNumber(DocumentSerialNumber);

    /// <summary>
    /// Verifies the Rhino document is available for use
    /// </summary>
    public bool DocumentIsAvailable
    {
      get
      {
        var doc = Document;
        if (null != doc && doc.IsAvailable)
          return true;
        return false;
      }
    }

    /// <summary>
    /// Use for binding to some label control
    /// </summary>
    public string DocumentLabel => "Document serial number: " + DocumentSerialNumber.ToString();

    #endregion

    /// <summary>
    /// Hooks up Rhino event handlers that we will leave running for the life of the view model
    /// </summary>
    private void HookRhinoEvents()
    {
      // TODO: add the events you want your panel to subscribed to here

      if (null == g_begin_command)
        Command.BeginCommand += g_begin_command = OnBeginCommand;

      if (null == g_end_command)
        Command.EndCommand += g_end_command = OnEndCommand;

    }

    /// <summary>
    /// Helper method to get view model from the document
    /// </summary>
    private static SampleDocumentPanelViewModel TryGetFromDocument(RhinoDoc document)
    {
      object value = null;
      document?.RuntimeData.TryGetValue(typeof(SampleDocumentPanelViewModel), out value);
      return value as SampleDocumentPanelViewModel;
    }

    /// <summary>
    /// Command.BeginCommand handler
    /// </summary>
    private static void OnBeginCommand(object sender, CommandEventArgs e)
    {
      var model = TryGetFromDocument(e.Document);
      if (null != model && model.DocumentIsAvailable)
      {
        Debug.WriteLine($"Begin {e.CommandEnglishName} command");
      }
    }

    /// <summary>
    /// Command.EndCommand handler
    /// </summary>
    private static void OnEndCommand(object sender, CommandEventArgs e)
    {
      var model = TryGetFromDocument(e.Document);
      if (null != model && model.DocumentIsAvailable)
      {
        Debug.WriteLine($"End {e.CommandEnglishName} command");
      }
    }
  }
}
